

/* EggCooker_v0 written by Tango2

This program is my attempt at creating a BBQ controller for my XL BGE using
an arduino to monitor grill and meat temps as well as control a fan to
regulate the grill temperature.

Components used:
- Arduino UNO
- 10CFM Brushless DC Blower
- Level Shifter
- K-type Thermocouple
- K-type Grill Probe
- K-type Meat Probe


Pin assignments:

*/

/*
Tasks:
- Selftest
  - Fan 0-100-0
  - Check for Probes

- Set Grill Temp

- Setup Display
  - Grill Temp
  - Probe 1
  - Probe 2
  - Fan Status
  - Elapsed Time

- Measure Temperature
  - Grill
  - Probe 1
  - Probe 2

- Analyze Temperature
  - PID Contol

- Control Fan
  - PWM Value
  - On/Off

*/

//-------------------------------- Classes ------------------------


//-------------------------------- Includes -----------------------

#include <MySensor.h>
#include <OneWire.h>
#include <DallasTemperature.h>
#include <SPI.h>
#include <PID_v1.h>

//-------------------------------- Defines ------------------------


#define oneWirePin 9
#define TempPrecision 9
#define MAX_ATTACHED_DS18B20 16

OneWire oneWire(oneWirePin); //Setup oneWire instance to communiate with any oneWire devices
DallasTemperature sensors(&oneWire); //Pass our oneWire referenceto Dallas Temperature.
DeviceAddress Probe0, Probe1, Probe2, Probe3; //Define arrays to hold device addresses.
MySensor gw;
unsigned long SLEEP_TIME = 15000;
float lastTemperature[MAX_ATTACHED_DS18B20];
int numSensors = 3;
boolean receivedConfig = false;
boolean metric = true;
MyMessage msg(0,V_TEMP);

void sketchInfo () {
  gw.begin();
  gw.sendSketchInfo("SmokerStoker","1.0");
}

void setupProbes () {
  sensors.begin();
  sensors.getAddress(Probe0, 0);
  sensors.getAddress(Probe1, 1);
  sensors.setResolution(Probe0, TempPrecision);
  sensors.setResolution(Probe1, TempPrecision);
}

// Cycle fan from 0-100-0% power over 10 seconds.
void selftestFan () {

}

// Verify which probes are connected and report status for 10 seconds
void selftestProbes () {

}

void printTemperature(DeviceAddress deviceAddress) {
  float tempC = sensors.getTempC(deviceAddress);
  //Display Temp in °C
  //Serial.print("Temp C: ");
  //Serial.print(tempC);
  //Display Temp in °F
  Serial.print(DallasTemperature::toFahrenheit(tempC));
  Serial.print(" F ");
}

void printData(DeviceAddress deviceAddress) {
  //Serial.print("Device Address: ");
  //printAddress(deviceAddress);
  //Serial.print(" ");
  printTemperature(deviceAddress);
  //Serial.println();
}

void sendTemps () {
  for (int i=0; i<numSensors && i<MAX_ATTACHED_DS18B20; i++) {
    float temperature = static_cast<float>(static_cast<int>((gw.getConfig().isMetric?sensors.getTempCByIndex(i):sensors.getTempFByIndex(i)) * 10.)) /10.;
    gw.send(msg.setSensor(i).set(temperature,1));
    lastTemperature[i]=temperature;
  }
 gw.sleep(SLEEP_TIME);
}
  
void setup() {

  Serial.begin(9600);
  sketchInfo();
  setupProbes();
  //selftestFan();
  //selftestLCD();
}

void loop() {
  // put your main code here, to run repeatedly:

  gw.process();
  sensors.requestTemperatures();
  printTemperature(Probe1);
  printTemperature(Probe2);
  sendTemps();
  //readTemps(Probe0);
  //readTemps(Probe1);
}

